/*
 * TicketAgent.java
 *
 * Created on January 21, 2002, 4:37 PM
 */

package unleashed.ch3;

import unleashed.TicketRequest2;
import java.io.*;
import java.util.*;
import java.sql.*;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;

import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

import org.w3c.dom.Document;
import org.w3c.dom.DOMException;
import org.w3c.dom.Node;
import org.w3c.dom.*;
/**
 * The TicketAgent application.  It received an XML TicketRequest file
 * It uses JDBC to access the database to
 * get the credit card number.It then sends a message to the PaymentAgent
 * to request approval of a charge to the credit card.  When the
 * charge is approved TicketAgent updates the database and sends
 * a message to the CruiseList application.
 *
 *
 * @author Steve Potts
 */
public class TicketAgent
{
    static int ticketID = 7000;
    private boolean quit = false;
    //JDBC variables
    java.sql.Connection dbConn = null;
    Statement statement1 = null;
    String sourceURL = "jdbc:odbc:CruiseTicket";
    
    DocumentBuilderFactory factory;
    DocumentBuilder builder;
    Document document;
    
    TicketRequest2 tr2;
    String eleName;
    
    
    
    public TicketAgent() throws Exception
    {
        init();
    }
    
    //The init() method moves processing out of the constructors where
    //Exception handling is simpler
    private void init() throws Exception
    {
        try
        {
            //Obtain connections to JDBC and JMS via JNDI
            ConnectToServices();
            
        }catch(Exception e)
        {
            System.out.println("Exception thrown " + e);
        }
    }
    
    private void ConnectToServices()
    {
        try
        {
            // ============== Make connection to database ==============
            
            //load the driver class
            Class.forName("sun.jdbc.odbc.JdbcOdbcDriver");
            
            
            //connect to the XML parser factory and builder
            factory = DocumentBuilderFactory.newInstance();
            builder = factory.newDocumentBuilder();
            
            
        } catch (Exception e)
        {
            System.out.println("Exception was thrown: " + e);
        }
    }
    
    public void checkForXMlDocs()
    {
        String dirName = "c:/XML/request/";
        File dir = new File(dirName);
        File f1;
        
        String[]  fileList = dir.list();
        for (int i=0;i<fileList.length;++i)
        {
            String s = fileList[i];
            if (s.startsWith("r"))
            {
                System.out.println(fileList[i]);
                parseFile(dirName + fileList[i]);
                System.out.println(tr2);
            }
        }
        
    }
    
    public void parseFile(String fname)
    {
        try
        {
            tr2 = new TicketRequest2();
            File f1 = new File(fname);
            document = builder.parse(f1);
            traverse(document);
            
            //get the credit card number from the database
            String ccNum = getCreditCardNumber(tr2.getCustID());
            System.out.println("The Credit Card Number is " + ccNum);
            
            //create a queue send to the Payment queue
            if (verifyCreditCard(ccNum))
            {
                createTicket(tr2);
                sendTicketResponse(tr2, true);
            }else
                sendTicketResponse(tr2, false);
            
        }catch (SAXException sxe)
        {
            Exception e = sxe;
            if (sxe.getException() != null)
                e = sxe.getException();
            e.printStackTrace();
        }
        catch (ParserConfigurationException pce)
        {
            pce.printStackTrace();
        }
        catch (IOException ioe)
        {
            ioe.printStackTrace();
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }//parseFile
    
    private void traverse(Node cNode)
    {
        switch (cNode.getNodeType() )
        {
            case Node.DOCUMENT_NODE:
                System.out.println("Element " + cNode.getNodeName());
                processChildren( cNode.getChildNodes());
                break;
                
            case Node.ELEMENT_NODE:
                eleName = cNode.getNodeName();
                System.out.println("Element " + eleName);
                NamedNodeMap attributeMap = cNode.getAttributes();
                int numAttrs = attributeMap.getLength();
                for (int i=0; i<attributeMap.getLength(); i++)
                {
                    Attr attribute = (Attr)attributeMap.item(i);
                    String attrName =  attribute.getNodeName();
                    String attrValue = attribute.getNodeValue();
                    storeElementValue(attrName, attrValue);
                }
                if (eleName.equals("isCommissionable"))
                {
                    storeElementValue("isCommissionable", "");
                }
                
                processChildren( cNode.getChildNodes());
                break;
            case Node.CDATA_SECTION_NODE:
            case Node.TEXT_NODE:
                
                System.out.println("Text " + cNode.getNodeValue());
                if (! cNode.getNodeValue().trim().equals(""))
                {
                    System.out.println("eleName " + eleName);
                    System.out.println("Text " + cNode.getNodeValue());
                    storeElementValue(eleName, cNode.getNodeValue());
                }
                break;
        }
    }
    
    private void processChildren(NodeList nList)
    {
        if(nList.getLength() != 0)
        {
            for (int i=0; i<nList.getLength(); i++)
                traverse(nList.item(i));
        }
    }
    
    
    private void storeElementValue(String elementName,
                                   String elementValue)
    {
        if (elementName.equals("ticketRequest"))
        {
        }
        
        if (elementName.equals("custID"))
        {
            tr2.setCustID(Integer.parseInt(elementValue));
        }
        
        if (elementName.equals("lastName"))
        {
            tr2.setLastName(elementValue);
        }
        
        if (elementName.equals("firstName"))
        {
            tr2.setFirstName(elementValue);
        }
        
        if (elementName.equals("cruiseID"))
        {
            tr2.setCruiseID(Integer.parseInt(elementValue));
        }
        
        if (elementName.equals("destination"))
        {
            tr2.setDestination(elementValue);
        }
        
        if (elementName.equals("port"))
        {
            tr2.setPort(elementValue);
        }
        
        if (elementName.equals("sailing"))
        {
            tr2.setSailing(elementValue);
        }
        
        if (elementName.equals("numberOfTickets"))
        {
            String numberOfTicketsString = elementValue;
            int numberOfTickets = 
                Integer.parseInt(numberOfTicketsString);
            tr2.setNumberOfTickets(numberOfTickets);
        }
        
        if (elementName.equals("isCommissionable"))
        {
            tr2.setCommissionable(true);
        }
    }
    
    public String toString()
    {
        return tr2.toString();
    }
    
    
    //This method sends a message to the response queue
    private void sendTicketResponse(TicketRequest2 tr,
                                     boolean isSuccessful)
    {
        try
        {
            //Create a message using this object
            String tickResp = "The ticket to " + tr.getDestination() +
             " for " + tr.getLastName();
            if (isSuccessful)
                tickResp += " was successful";
            else
                tickResp += " was not successful";
            
            System.out.println(tickResp);
            
            //Write the response
            char quote = '"';
            BufferedWriter bw = new BufferedWriter(
            new FileWriter("c:/XML/response/response"
            + tr2.getCustID() + ".xml"));
            
            bw.write(System.getProperty("line.separator"));
            
            bw.write("<ticketResponse>");
            bw.write(System.getProperty("line.separator"));
            bw.write(tickResp);
            bw.write(System.getProperty("line.separator"));
            bw.write("</ticketResponse>");
            bw.flush();
            bw.close();
            
            System.out.println("Created  the Ticket Response Message");
            
        }catch(Exception e)
        {
            System.out.println("Exception " + e);
        }
    }
    
    //This method creates a database entry for the new ticket
    private void createTicket(TicketRequest2 tr) throws Exception
    {
        //get a connection to the database
        dbConn =DriverManager.getConnection(sourceURL);
        
        //If we get to here, no exception was thrown
        System.out.println("The database connection is " + dbConn);
        System.out.println("Making connection...\n");
        
        //Create the statement
        statement1 = dbConn.createStatement();
        
        String insertStatement;
        ticketID += 1;
        insertStatement = "INSERT INTO CruiseTicket VALUES("
        + ticketID + "," +
        Integer.toString(tr.getCustID()) + ", 'Unleashed Cruise Line'," +
        "'USS SeaBuscuit', '" + tr.getPort() + "','" + tr.getSailing() +
        "'," + "999.99," + "0,"+ "0,"+ "'')";
        
        statement1.executeUpdate(insertStatement);
        System.out.println("Update was successful CustID = " +
                             tr.getCustID());
        
        statement1.close();
        dbConn.close();
        
    }
    
    private String getCreditCardNumber(int CustID) throws Exception
    {
        //get a connection to the database
        dbConn =DriverManager.getConnection(sourceURL);
        
        //If we get to here, no exception was thrown
        System.out.println("The database connection is " + dbConn);
        System.out.println("Making connection...\n");
        
        //Create the statement
        statement1 = dbConn.createStatement();
        
        String ccNum = "";
        
        //Populate the creditcard string string
        String getCCString =
        "SELECT CreditCardNumber FROM CruiseCustomer " +
        "WHERE CustomerID = " + Integer.toString(CustID);
        
        ResultSet ccResults = statement1.executeQuery(getCCString);
        
        while (ccResults.next())
        {
            ccNum = ccResults.getString("CreditCardNumber");
        }
        
        statement1.close();
        dbConn.close();
        return ccNum;
    }
    
    private boolean verifyCreditCard(String ccNum)
    {
        int firstNum = Integer.parseInt(ccNum.substring(0,1));
        
        if ( firstNum > 4)
            return false;
        else
            return true;
    }
    
    /**
     * main() method.
     *
     * @exception  Exception if execution fails
     */
    public static void main(String[] args) throws Exception
    {
        try
        {
            TicketAgent ta = new TicketAgent();
            
            ta.checkForXMlDocs();
            
        }catch (Exception e)
        {
            System.out.println("Exception in main() " + e);
        }
    }
}