/*
 * SimpleQueueClient.java
 *
 * Created on January 9, 2002, 7:39 PM
 */


package unleashed.ch6.SimpleQueue;

import java.rmi.RemoteException;
import java.util.Properties;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.jms.Queue;
import javax.jms.QueueConnection;
import javax.jms.QueueConnectionFactory;
import javax.jms.QueueSender;
import javax.jms.QueueSession;
import javax.ejb.CreateException;
import javax.ejb.RemoveException;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;

/**
 * This class illustrates calling a Message-Driven bean and publishing
 * quotes on a Queue.
 *
 * @author Steve Potts
 */

public class SimpleQueueClient
{
    private Context context;
    private QueueConnection qConnection;
    private QueueSession qSession;
    private Queue simpleQueue;
    private TextMessage tm;
    
    
    public SimpleQueueClient() throws NamingException
    {
        try
        {
            context = getInitialContext();
            
            // Create the connection
            QueueConnectionFactory cf =
            (QueueConnectionFactory) context.lookup("weblogic.jms.ConnectionFactory");
            qConnection = cf.createQueueConnection();
            qConnection.start();
            
            
            //Get a handle to a QueueSession
            qSession =
            qConnection.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);
            simpleQueue = (Queue) context.lookup("SimpleQueue");
            QueueSender ts = qSession.createSender(simpleQueue);
            tm = qSession.createTextMessage("This is the first Queue message");
            ts.send(tm);
            log("q message1 sent");
            tm = qSession.createTextMessage("This is the second Queue message");
            ts.send(tm);
            log("q message2 sent");
            tm = qSession.createTextMessage("This is the third Queue message");
            ts.send(tm);
            log("q message3 sent");
            
        }catch (Exception e)
        {
            log("Exception " + e);
        }
    }
    
    //main()
    public static void main(String[] args) throws Exception
    {
        
        log("main() started");
        
        try
        {
            SimpleQueueClient stc = new SimpleQueueClient();
        }catch (Exception e)
        {
            log("Exception " + e);
        }
    }
    
    //get the jndi context
    private Context getInitialContext() throws NamingException
    {
        try
        {
            // Get an InitialContext
            Properties h = new Properties();
            h.put(Context.INITIAL_CONTEXT_FACTORY,
            "weblogic.jndi.WLInitialContextFactory");
            h.put(Context.PROVIDER_URL, "t3://127.0.0.1:7001");
            return new InitialContext(h);
        }
        catch (NamingException ex)
        {
            log("can't connect to the  server " + ex);
            throw ex;
        }
    }
    
    
    private static void log(String s)
    {
        System.out.println(s);
    }
    
}

