/*
 * SimpleTopic.java
 *
 * Created on January 9, 2002, 2:05 PM
 */

package unleashed.ch6.SimpleTopic;

import java.rmi.RemoteException;
import java.util.Properties;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.jms.Topic;
import javax.jms.TopicConnection;
import javax.jms.TopicConnectionFactory;
import javax.jms.TopicPublisher;
import javax.jms.TopicSession;
import javax.ejb.CreateException;
import javax.ejb.RemoveException;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;

/**
 * This class illustrates calling a Message-Driven bean and publishing
 * quotes on a topic.
 *
 * @author Steve Potts
 */

public class SimpleTopicClient
{
    private Context context;
    private TopicConnection tConnection;
    private TopicSession tSession;
    private Topic simpleTopic;
    private TextMessage tm;
    
    
    public SimpleTopicClient() throws NamingException
    {
        try
        {
            context = getInitialContext();
            
            // Create the connection
            TopicConnectionFactory cf =
            (TopicConnectionFactory) context.lookup("weblogic.jms.ConnectionFactory");
            tConnection = cf.createTopicConnection();
            tConnection.start();
            
            
            //Get a handle to a topicSession
            tSession =
            tConnection.createTopicSession(false, Session.AUTO_ACKNOWLEDGE);
            simpleTopic = (Topic) context.lookup("SimpleTopic");
            TopicPublisher tp = tSession.createPublisher(simpleTopic);
            tm = tSession.createTextMessage("This is the first message");
            tp.publish(tm);
            log("message1 sent");
            tm = tSession.createTextMessage("This is the second message");
            tp.publish(tm);
            log("message2 sent");
            tm = tSession.createTextMessage("This is the third message");
            tp.publish(tm);
            log("message3 sent");
            
        }catch (Exception e)
        {
            log("Exception " + e);
        }
    }
    
    //main()
    public static void main(String[] args) throws Exception
    {
        
        log("main() started");
        
        try
        {
            SimpleTopicClient stc = new SimpleTopicClient();
        }catch (Exception e)
        {
            log("Exception " + e);
        }
    }
    
    //get the jndi context
    private Context getInitialContext() throws NamingException
    {
        try
        {
            // Get an InitialContext
            Properties h = new Properties();
            h.put(Context.INITIAL_CONTEXT_FACTORY,
            "weblogic.jndi.WLInitialContextFactory");
            h.put(Context.PROVIDER_URL, "t3://127.0.0.1:7001");
            return new InitialContext(h);
        }
        catch (NamingException ex)
        {
            log("can't connect to the  server " + ex);
            throw ex;
        }
    }
    
    
    private static void log(String s)
    {
        System.out.println(s);
    }
    
}
