//Listing 7.2 The CruiseList class contains code for the 
//UserInterface as well as code for creating the TicketRequest.
/*
 * CruiseList.java
 *
 * Created on December 31, 2001, 6:35 PM
 */

package unleashed.ch7;
import javax.swing.*;
import java.awt.event.ActionListener;
import javax.swing.border.EtchedBorder;
import java.awt.Container;
import java.awt.BorderLayout;
import java.sql.*;
import javax.naming.*;
import javax.jms.*;
import java.util.*;
import unleashed.TicketRequest2;

/**
 *
 * @author  Stephen Potts
 * @version
 */
public class CruiseList extends JFrame implements ActionListener,
                                                  MessageListener
{
    //JDBC variables
    java.sql.Connection dbConn = null;
    Statement statement1 = null;
    
    //JMS variables
    QueueConnectionFactory qconFactory;
    QueueConnection qcon;
    QueueSession qsession;
    QueueSender qTicketRequestSender;
    QueueReceiver qTicketResponseReceiver;
    Queue ticketRequestQ;
    Queue ticketResponseQ;
    
    //GUI Variables
    JList customerList;
    JList cruisesAvailable;
    JButton btnBook;
    JButton btnExit;
    
    //Arrays to hold database information
    int[] custNums = null;
    String[] lastNames = null;
    String[] firstNames = null;
    
    int[] cruiseIDs = null;
    String[] cruiseDestinations = null;
    String[] cruisePorts = null;
    String[] cruiseSailings = null;
    
    /** Constructors for CruiseList */
    public CruiseList() throws Exception
    {
        init();
    }
    
    public CruiseList(String caption) throws Exception
    {
        super(caption);
        init();
    }
    
    //The init() method moves real processing out of the constructors 
    //Exception handling is simpler
    private void init() throws Exception
    {
        try
        {
            //Obtain connections to JDBC and JMS via JNDI
            ConnectToServices();
            
            //configure the Frame
            setBounds(150,200,500,250);
            setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
            
            //set the layout
            BorderLayout border = new BorderLayout();
            Container content = getContentPane();
            content.setLayout(border);
            
            //Populate the customerNames string
            String getCustomerString =
            "SELECT CustomerID, LastName, FirstName FROM CruiseCustomer";
            ResultSet custResults = 
                    statement1.executeQuery(getCustomerString);
            
            custNums = new int[20];
            lastNames = new String[20];
            firstNames = new String[20];
            int index = 0;
            
            while (custResults.next())
            {
                custNums[index] = custResults.getInt("CustomerID");
                firstNames[index] = custResults.getString("FirstName");
                lastNames[index] = custResults.getString("LastName");
                index += 1;
            }
            
            String[] customerNames = new String[index];
            
            for (int i=0;i<index;i++)
            {
                customerNames[i] = firstNames[i] + " " + lastNames[i];
            }
            int numCustomers = index;
            System.out.println("The number of customers " + index);
            
            //Populate the cruises string
            String getCruiseString =
            "SELECT CruiseID, Destination, Port, Sailing FROM Cruises";
            ResultSet cruiseResults = 
                       statement1.executeQuery(getCruiseString);
            
            cruiseIDs = new int[20];
            cruiseDestinations = new String[20];
            cruisePorts = new String[20];
            cruiseSailings = new String[20];
            index = 0;
            
            while (cruiseResults.next())
            {
                cruiseIDs[index] = cruiseResults.getInt("CruiseID");
                cruiseDestinations[index] = 
                             cruiseResults.getString("Destination");
                cruisePorts[index] = cruiseResults.getString("Port");
                cruiseSailings[index] =
                                cruiseResults.getString("Sailing");
                index += 1;
            }
            
            String[] cruises = new String[index];
            
            for (int i=0;i<index;i++)
            {
                cruises[i] = cruiseDestinations[i] +
                                "      Departs: " + cruisePorts[i] +
                 " " + cruiseSailings[i];
            }
            
            int numCruises = index;
            System.out.println("The number of cruises" + index);
            
            
            //More GUI components
            String labelString = "               Customer";
            labelString += "                          Cruise";
            JLabel label1 = new JLabel(labelString);
            customerList = new JList(customerNames);
            cruisesAvailable = new JList(cruises);
            btnBook = new JButton("Book");
            btnExit = new JButton("Exit");
            btnBook.addActionListener(this);
            btnExit.addActionListener(this);
            
            JPanel bottomPanel = new JPanel();
            JPanel centerPanel = new JPanel();
            centerPanel.add(new JScrollPane(customerList, 
            JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
            JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED));
            centerPanel.add(new JScrollPane(cruisesAvailable, 
            JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
            JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED));
            bottomPanel.add(btnBook);
            bottomPanel.add(btnExit);
            content.add(label1, BorderLayout.NORTH);
            content.add(centerPanel, BorderLayout.CENTER);
            content.add(bottomPanel, BorderLayout.SOUTH);
            setVisible(true);
        }catch(Exception e)
        {
            System.out.println("Exception thrown " + e);
        } finally
        {
            try
            {
                //close all connection
                if (statement1 != null)
                    statement1.close();
                if (dbConn != null)
                    dbConn.close();
                
            } catch (SQLException sqle)
            {
                System.out.println("SQLException during close(): " +
                                                 sqle.getMessage());
            }
        }
        
    }
    
    private void ConnectToServices()
    {
        try
        {
            // Make connection to database
            // Obtain a Datasource connection from JNDI
            Context ctx = null;
            
            // Put connection properties in a hashtable.
            Hashtable ht = new Hashtable();
            ht.put(Context.INITIAL_CONTEXT_FACTORY,
            "weblogic.jndi.WLInitialContextFactory");
            ht.put(Context.PROVIDER_URL,
            "t3://localhost:7001");
            
            // Get a context for the JNDI look up
            ctx = new InitialContext(ht);
            
            //Get a connection to the datasource
            javax.sql.DataSource ds
            = (javax.sql.DataSource) ctx.lookup(
                                       "examples-dataSource-demoPool");
            dbConn = ds.getConnection();
            System.out.println("Making Database connection...\n");
            
            //Create the statement
            statement1 = dbConn.createStatement();
            
            //Get a connection to the naming service
            qconFactory = (QueueConnectionFactory)
            ctx.lookup("weblogic.examples.jms.QueueConnectionFactory");
            qcon = qconFactory.createQueueConnection();
            qsession = 
              qcon.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);
            ticketRequestQ = (Queue)ctx.lookup("TicketRequestQueue2");
            ticketResponseQ = (Queue)ctx.lookup("TicketResponseQueue2");
            
            //create a sender
            qTicketRequestSender = qsession.createSender(ticketRequestQ);
            qTicketResponseReceiver = 
                         qsession.createReceiver(ticketResponseQ);
            qTicketResponseReceiver.setMessageListener(this);

            qcon.start();
            System.out.println("JMS Connections created");
            
        } catch (Exception e)
        {
            System.out.println("Exception was thrown: " + e);
        }
    }
    
    /**
     * @param args the command line arguments
     */
    public static void main(String args[])
    {
        //create an instance of the GUI
        try
        {
            CruiseList mainWindow =
                     new CruiseList("Cruise Ticket System");
        }catch(Exception e)
        {
            System.out.println("Exception in main " + e);
        }
    }
    
    public void actionPerformed(java.awt.event.ActionEvent ae)
    {
        try
        {
            Container c = btnExit.getParent();
            if (ae.getActionCommand().equals("Exit"))
            {
                //close all JMS Connections
                try
                {
                    if (qcon != null)
                        qcon.close();
                    if (qsession != null)
                        qsession.close();
                    if (qTicketRequestSender != null)
                        qTicketRequestSender.close();
                    if (qTicketResponseReceiver != null)
                        qTicketResponseReceiver.close();
                }catch(Exception e)
                {
                    System.out.println("Exception in main " + e);
                }
                System.exit(0);
            }
            
            //Try and book a ticket
            if (ae.getActionCommand().equals("Book"))
            {
                System.out.println("Book was clicked");
                int custIndex = customerList.getSelectedIndex();
                int cruiseIndex = cruisesAvailable.getSelectedIndex();
                
                if (custIndex == -1 || cruiseIndex == -1)
                {
                    JOptionPane.showMessageDialog(c,
                    "You must choose a customer and a cruise");
                }else
                {
                    //Pop up a dialog asking how many tickets 
                    String numTickets = JOptionPane.showInputDialog(c,
                    "How many tickets?");
                    int numberOfTickets = Integer.parseInt(numTickets);
                    
                    //create a ticket request object
                    TicketRequest2 tickReq = 
                     new TicketRequest2(custNums[custIndex],
                    lastNames[custIndex], firstNames[custIndex],
                    cruiseIDs[cruiseIndex], cruiseDestinations[cruiseIndex],
                    cruisePorts[cruiseIndex], cruiseSailings[cruiseIndex],
                    numberOfTickets, false);
                    
                    //System.out.println(tickReq);
                    //Create a message using this object
                    ObjectMessage oMessage =
                           qsession.createObjectMessage(tickReq);
                    System.out.println(
                          "Created  the Ticket Request Message");
                    
                    //put the message on the queue
                    qTicketRequestSender.send(oMessage);
                    System.out.println(
                         "Sent the ticket Request Message");
                }
            }
        }catch (Exception e)
        {
            System.out.println("Exception thrown = " + e);
        }
    }
    
    /**
     * Message listener interface.
     * @param msg  message
     */
    
    // MessageListener interface
    public void onMessage(Message msg)
    {
        Container c = btnExit.getParent();
        try
        {
            //read the text message from JMS
            String msgText = "";
            if (msg instanceof TextMessage)
                msgText = ((TextMessage)msg).getText();
            else
                msgText = msg.toString();
            System.out.println(msgText);
            
            //show the message in a dialog box
            JOptionPane.showMessageDialog(c, msgText);
        }catch (Exception e)
        {
            System.out.println("onMessage() Exception " + e);
        }
    }
}

