//Listing 7.4 The DealAnnouncer uses a messaging system to send the same message to //several applications at once. 
/*
 * DealAnnouncer.java
 *
 * Created on January 4, 2002, 6:55 PM
 */

package unleashed.ch7;

import javax.swing.*;
import java.awt.event.ActionListener;
import javax.swing.border.EtchedBorder;
import java.awt.Container;
import java.awt.BorderLayout;
import javax.naming.*;
import javax.jms.*;
import java.util.*;

/**
 *
 * @author  Stephen Potts
 * @version
 */
public class DealAnnouncer extends JFrame implements ActionListener
{
    
    //JMS variables
    TopicConnectionFactory tConFactory;
    TopicConnection tCon;
    TopicSession tSession;
    TopicPublisher tDealPublisher;
    Topic dealAnnounceTopic;
    JButton btnPublish;
    JButton btnExit;
    JTextArea jta;
    
    /** Constructors for CruiseList */
    public DealAnnouncer() throws Exception
    {
        init();
    }
    
    public DealAnnouncer(String caption) throws Exception
    {
        super(caption);
        init();
    }
    
    //The init() method moves real processing out of the constructors where
    //Exception handling is simpler
    private void init() throws Exception
    {
        try
        {
            //Obtain connections to JMS via JNDI
            ConnectToServices();
            
            //configure the Frame
            setBounds(150,200,500,250);
            setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
            
            //set the layout
            BorderLayout border = new BorderLayout();
            Container content = getContentPane();
            content.setLayout(border);
            
            
            //More GUI components
            String labelString = "                                    " + 
            "                          Message";
            JLabel label1 = new JLabel(labelString);
            btnPublish = new JButton("Publish");
            btnExit = new JButton("Exit");
            btnPublish.addActionListener(this);
            btnExit.addActionListener(this);
            
            JPanel bottomPanel = new JPanel();
            JPanel centerPanel = new JPanel();
            jta = new JTextArea(15,30);
            jta.setLineWrap(true);
            jta.setWrapStyleWord(true);
            centerPanel.add(new JScrollPane(jta,
            JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
            JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED));
            bottomPanel.add(btnPublish);
            bottomPanel.add(btnExit);
            content.add(label1, BorderLayout.NORTH);
            content.add(centerPanel, BorderLayout.CENTER);
            content.add(bottomPanel, BorderLayout.SOUTH);
            setVisible(true);
        }catch(Exception e)
        {
            System.out.println("Exception thrown " + e);
        }
    }
    
    private void ConnectToServices()
    {
        try
        {
            // Obtain a Datasource connection from JNDI
            Context ctx = null;
            
            // Put connection properties in a hashtable.
            Hashtable ht = new Hashtable();
            ht.put(Context.INITIAL_CONTEXT_FACTORY,
            "weblogic.jndi.WLInitialContextFactory");
            ht.put(Context.PROVIDER_URL,
            "t3://localhost:7001");
            
            // Get a context for the JNDI look up
            ctx = new InitialContext(ht);
            
            
            //Get a connection to the naming service
            tConFactory = (TopicConnectionFactory)
            ctx.lookup("weblogic.examples.jms.TopicConnectionFactory");
            tCon = tConFactory.createTopicConnection();
            tSession = tCon.createTopicSession(false, 
                                   Session.AUTO_ACKNOWLEDGE);
            dealAnnounceTopic = (Topic)ctx.lookup("dealTopic");
            
            //create a sender
            tDealPublisher = tSession.createPublisher(dealAnnounceTopic);
            tCon.start();
            System.out.println("JMS Connections created");
            
        } catch (Exception e)
        {
            System.out.println("Exception was thrown: " + e);
        }
    }
    
    /**
     * @param args the command line arguments
     */
    public static void main(String args[])
    {
        //create an instance of the GUI
        try
        {
            DealAnnouncer mainWindow =
                new DealAnnouncer("Deal Announcer System");
        }catch(Exception e)
        {
            System.out.println("Exception in main " + e);
        }
    }
    
    public void actionPerformed(java.awt.event.ActionEvent ae)
    {
        try
        {
            Container c = btnExit.getParent();
            if (ae.getActionCommand().equals("Exit"))
            {
                //close all JMS Connections
                try
                {
                    if (tCon != null)
                        tCon.close();
                    if (tSession != null)
                        tSession.close();
                    if (tDealPublisher != null)
                        tDealPublisher.close();
                }catch(Exception e)
                {
                    System.out.println("Exception in main " + e);
                }
                System.exit(0);
            }
            
            //Try and book a ticket
            if (ae.getActionCommand().equals("Publish"))
            {
                System.out.println("Publish was clicked");
                String message1 = jta.getText();
                JOptionPane.showMessageDialog(c,
                message1);
                
                
                //Create a message using this object
                TextMessage message =
                          tSession.createTextMessage(message1);
                System.out.println(
                       "Created  the Cruise Discount Message");
                
                //put the message on the Topic
                tDealPublisher.publish(message);
                System.out.println(
                      "Published the Cruise Discount Message");
            }
            
        }catch (Exception e)
        {
            System.out.println("Exception thrown = " + e);
        }
    }
}
