
//Listing 7.5 Topics have subscribers who get notification who receive messages posted to //it.
/*
 * DealSubscriber.java
 *
 * Created on January 4, 2002, 8:17 PM
 */

package unleashed.ch7;
import javax.swing.*;
import java.awt.event.ActionListener;
import javax.swing.border.EtchedBorder;
import java.awt.Container;
import java.awt.BorderLayout;
import java.sql.*;
import javax.naming.*;
import javax.jms.*;
import java.util.*;

/**
 *
 * @author  Stephen Potts
 * @version
 */
public class DealSubscriber extends JFrame implements ActionListener,
                                                      MessageListener
{
    
    //JMS variables
    //JMS variables
    TopicConnectionFactory tConFactory;
    TopicConnection tCon;
    TopicSession tSession;
    TopicSubscriber tDealSubscriber;
    Topic dealAnnounceTopic;
    
    //GUI Variables
    JButton btnExit;
    
    
    /** Constructors for CruiseList */
    public DealSubscriber() throws Exception
    {
        init();
    }
    
    public DealSubscriber(String caption) throws Exception
    {
        super(caption);
        init();
    }
    
    //The init() method moves real processing out of the constructors
    //Exception handling is simpler
    private void init() throws Exception
    {
        try
        {
            //Obtain connections to JDBC and JMS via JNDI
            ConnectToServices();
            
            //configure the Frame
            setBounds(150,200,500,250);
            setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
            
            //set the layout
            BorderLayout border = new BorderLayout();
            Container content = getContentPane();
            content.setLayout(border);
            
            //More GUI components
            String labelString = "               Customer";
            labelString += "                          Cruise";
            JLabel label1 = new JLabel(labelString);
            btnExit = new JButton("Exit");
            btnExit.addActionListener(this);
            
            JPanel bottomPanel = new JPanel();
            JPanel centerPanel = new JPanel();
            bottomPanel.add(btnExit);
            content.add(label1, BorderLayout.NORTH);
            content.add(centerPanel, BorderLayout.CENTER);
            content.add(bottomPanel, BorderLayout.SOUTH);
            setVisible(true);
        }catch(Exception e)
        {
            System.out.println("Exception thrown " + e);
        }
        
    }
    
    private void ConnectToServices()
    {
        try
        {
            // Make connection to database
            // Obtain a Datasource connection from JNDI
            Context ctx = null;
            
            // Put connection properties in a hashtable.
            Hashtable ht = new Hashtable();
            ht.put(Context.INITIAL_CONTEXT_FACTORY,
            "weblogic.jndi.WLInitialContextFactory");
            ht.put(Context.PROVIDER_URL,
            "t3://localhost:7001");
            
            // Get a context for the JNDI look up
            ctx = new InitialContext(ht);
            
            //Get a connection to the naming service
            tConFactory = (TopicConnectionFactory)
            ctx.lookup("weblogic.examples.jms.TopicConnectionFactory");
            tCon = tConFactory.createTopicConnection();
            tSession = tCon.createTopicSession(false,
                        Session.AUTO_ACKNOWLEDGE);
            dealAnnounceTopic = (Topic)ctx.lookup("dealTopic");
            
            //create a listener
            tDealSubscriber =
                     tSession.createSubscriber(dealAnnounceTopic);
            tDealSubscriber.setMessageListener(this);
            tCon.start();
            System.out.println("JMS Connections created");
            
        } catch (Exception e)
        {
            System.out.println("Exception was thrown: " + e);
        }
    }
    
    /**
     * @param args the command line arguments
     */
    public static void main(String args[])
    {
        //create an instance of the GUI
        try
        {
            DealSubscriber mainWindow = 
                   new DealSubscriber("Deal Subscriber System");
        }catch(Exception e)
        {
            System.out.println("Exception in main " + e);
        }
    }
    
    public void actionPerformed(java.awt.event.ActionEvent ae)
    {
        try
        {
            Container c = btnExit.getParent();
            if (ae.getActionCommand().equals("Exit"))
            {
                //close all JMS Connections
                try
                {
                    if (tCon != null)
                        tCon.close();
                    if (tSession != null)
                        tSession.close();
                }catch(Exception e)
                {
                    System.out.println("Exception in main " + e);
                }
                System.exit(0);
            }
        }catch (Exception e)
        {
            System.out.println("Exception thrown = " + e);
        }
    }
    
    /**
     * Message listener interface.
     * @param msg  message
     */
    
    // MessageListener interface
    public void onMessage(Message msg)
    {
        Container c = btnExit.getParent();
        try
        {
            //read the text message from JMS
            String msgText = "";
            if (msg instanceof TextMessage)
                msgText = ((TextMessage)msg).getText();
            else
                msgText = msg.toString();
            System.out.println(msgText);
            
            //show the message in a dialog box
            JOptionPane.showMessageDialog(c, msgText);
        }catch (Exception e)
        {
            System.out.println("onMessage() Exception " + e);
        }
    }
}
