/*
 * BindCompany.java
 *
 * Created on January 28, 2002, 2:35 PM
 */

//Listing 11.10 The BindCompany class tests the serialization of this
// complex set of classes

package unleashed.ch11;

import java.io.ObjectOutputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Iterator;
import java.util.Hashtable;
import javax.naming.*;


/**
 *
 * @author  Stephen Potts
 * Test to see if company is serializable 
 */
public class BindCompany
{

    /** Creates new BindCompany */
    public BindCompany() {
    }
    
    public static void printCompanyObject(Company c)
    {
        System.out.println("The company name is " + c.getName());
        System.out.println("The company president is " + c.getPresident().getName());
        System.out.println(" ");

        Iterator i = c.getDepartmentIterator();
        while (i.hasNext())
        {
            Department d = (Department)i.next();
            System.out.println("   The department name is " + d.getName());
            System.out.println("   The department manager is " + d.getManager().getName());
            System.out.println(" ");
        }
    }
            
        
        
    public static void main(String[] args)
    {
        try
        {
           System.out.println("The Java Unleashed BindCompany");
     
            //Specify the URL of the naming service provider.  In this
            //case it is my computer
            String url = "t3://localhost:7001";
     
            //Use the hashtable approach to getting the InitialContext
            Hashtable env = new Hashtable();
     
            //Specify that we want the WebLogic context factory to be run
            env.put(Context.INITIAL_CONTEXT_FACTORY,
             "weblogic.jndi.WLInitialContextFactory");
            env.put(Context.PROVIDER_URL, url);
     
            //Create the InitialContext
            InitialContext ic = new InitialContext(env);

            
            Company comp = new Company("The Blazer Company");
            Employee emp0 = new Employee("Leslie Waller");
            comp.addPresident(emp0);
        
            Department sales = new Department("Sales");
            Employee emp1 = new Employee("Grant Jackson");
            sales.addManager(emp1);
            comp.addDepartment(sales);
            
            Department accounting = new Department("Accounting");
            Employee emp2 = new Employee("Clay Cain");
            accounting.addManager(emp2);
            comp.addDepartment(accounting);
           
            Department maintenance = new Department("Maintenance");
            Employee emp3 = new Employee("Greg Hladlick");
            maintenance.addManager(emp3);
            comp.addDepartment(maintenance);
            printCompanyObject(comp);


	      //Bind the String Object to the name "XYZ Company"
	      ic.bind("XYZ Company", comp);
            
           
        }catch (NamingException ne)
        {
            System.out.println("NamingException thrown " + ne);
        }
    }
}
