/*
 * CruiseList.java
 *
 * Created on January 14, 2002, 4:02 PM
 */

package unleashed.ch13;

import javax.swing.*;
import java.awt.event.ActionListener;
import javax.swing.border.EtchedBorder;
import java.awt.Container;
import java.awt.BorderLayout;
import java.sql.*;
import javax.naming.*;
import javax.mail.*;
import javax.mail.internet.*;
import java.util.*;
import java.io.*;

/**
 *
 * @author  Stephen Potts
 * @version
 */
public class CruiseList extends JFrame implements ActionListener
{
    //JDBC variables
    java.sql.Connection dbConn = null;
    Statement statement1 = null;
    
    //JavaMail variables
    String smtpServer;
    String to;
    String from;
    String subject;
    String body;
    Session session;
    
    String server;
    String user;
    String password;
    
    
    //GUI Variables
    JList customerList;
    JList cruisesAvailable;
    JButton btnBook;
    JButton btnCheck;
    JButton btnExit;
    
    //Arrays to hold database information
    int[] custNums = null;
    String[] lastNames = null;
    String[] firstNames = null;
    
    int[] cruiseIDs = null;
    String[] cruiseDestinations = null;
    String[] cruisePorts = null;
    String[] cruiseSailings = null;
    
    /** Constructors for CruiseList */
    public CruiseList() throws Exception
    {
        init();
    }
    
    public CruiseList(String caption) throws Exception
    {
        super(caption);
        init();
    }
    
    //The init() method moves real processing out of the constructors where
    //Exception handling is simpler
    private void init() throws Exception
    {
        try
        {
            //Obtain connections to JDBC and JMS via JNDI
            ConnectToServices();
            
            //configure the Frame
            setBounds(150,200,500,250);
            setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
            
            //set the layout
            BorderLayout border = new BorderLayout();
            Container content = getContentPane();
            content.setLayout(border);
            
            //Populate the customerNames string
            String getCustomerString =
            "SELECT CustomerID, LastName, FirstName FROM CruiseCustomer";
            ResultSet custResults = statement1.executeQuery(getCustomerString);
            
            custNums = new int[20];
            lastNames = new String[20];
            firstNames = new String[20];
            int index = 0;
            
            while (custResults.next())
            {
                custNums[index] = custResults.getInt("CustomerID");
                firstNames[index] = custResults.getString("FirstName");
                lastNames[index] = custResults.getString("LastName");
                index += 1;
            }
            
            String[] customerNames = new String[index];
            
            for (int i=0;i<index;i++)
            {
                customerNames[i] = firstNames[i] + " " + lastNames[i];
            }
            int numCustomers = index;
            System.out.println("The number of customers " + index);
            
            //Populate the cruises string
            String getCruiseString =
            "SELECT CruiseID, Destination, Port, Sailing FROM Cruises";
            ResultSet cruiseResults = statement1.executeQuery(getCruiseString);
            
            cruiseIDs = new int[20];
            cruiseDestinations = new String[20];
            cruisePorts = new String[20];
            cruiseSailings = new String[20];
            index = 0;
            
            while (cruiseResults.next())
            {
                cruiseIDs[index] = cruiseResults.getInt("CruiseID");
                cruiseDestinations[index] = cruiseResults.getString("Destination");
                cruisePorts[index] = cruiseResults.getString("Port");
                cruiseSailings[index] = cruiseResults.getString("Sailing");
                index += 1;
            }
            
            String[] cruises = new String[index];
            
            for (int i=0;i<index;i++)
            {
                cruises[i] = cruiseDestinations[i] + "      Departs: " + cruisePorts[i] + " " + cruiseSailings[i];
            }
            
            int numCruises = index;
            System.out.println("The number of cruises" + index);
            
            
            //More GUI components
            String labelString = "               Customer";
            labelString += "                          Cruise";
            JLabel label1 = new JLabel(labelString);
            customerList = new JList(customerNames);
            cruisesAvailable = new JList(cruises);
            btnBook = new JButton("Book");
            btnCheck = new JButton("Check");
            btnExit = new JButton("Exit");
            btnBook.addActionListener(this);
            btnCheck.addActionListener(this);
            btnExit.addActionListener(this);
            
            JPanel bottomPanel = new JPanel();
            JPanel centerPanel = new JPanel();
            centerPanel.add(new JScrollPane(customerList, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
            JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED));
            centerPanel.add(new JScrollPane(cruisesAvailable, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
            JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED));
            bottomPanel.add(btnBook);
            bottomPanel.add(btnCheck);
            bottomPanel.add(btnExit);
            content.add(label1, BorderLayout.NORTH);
            content.add(centerPanel, BorderLayout.CENTER);
            content.add(bottomPanel, BorderLayout.SOUTH);
            setVisible(true);
        }catch(Exception e)
        {
            System.out.println("Exception thrown " + e);
        } finally
        {
            try
            {
                //close all connection
                if (statement1 != null)
                    statement1.close();
                if (dbConn != null)
                    dbConn.close();
                
            } catch (SQLException sqle)
            {
                System.out.println("SQLException during close(): " + sqle.getMessage());
            }
        }
        
    }
    
    private void ConnectToServices()
    {
        try
        {
             
            //Get a connection to the datasource
             //load the driver class
            Class.forName("sun.jdbc.odbc.JdbcOdbcDriver");       
           
            //Specify the ODBC data source
            String sourceURL = "jdbc:odbc:CruiseTicket";
            
            //get a connection to the database
            dbConn =DriverManager.getConnection(sourceURL);
            
            //If we get to here, no exception was thrown
            System.out.println("The database connection is " + dbConn);
            System.out.println("Making connection...\n");
            
            //Create the statement
            statement1 = dbConn.createStatement();
            
            //Get a connection to the SMTP Service
            smtpServer = "mail.bellsouth.net";
            to = "sapotts@bellsouth.net";
            from = "sapotts@bellsouth.net";
            
            //Get a connection to POP
            server = "mail.bellsouth.net";
            user = "sapotts";
            password = "xyzxyz";
            
            
            Properties props = System.getProperties();
            
            props.put("mail.smtp.host", smtpServer);
            session = Session.getDefaultInstance(props, null);
            
            System.out.println("JavaMail Connections created");
            
        } catch (Exception e)
        {
            System.out.println("Exception was thrown: " + e);
        }
    }
    
    /**
     * @param args the command line arguments
     */
    public static void main(String args[])
    {
        //create an instance of the GUI
        try
        {
            CruiseList mainWindow = new CruiseList("Cruise Ticket System");
        }catch(Exception e)
        {
            System.out.println("Exception in main " + e);
        }
    }
    
    public void actionPerformed(java.awt.event.ActionEvent ae)
    {
        try
        {
            Container c = btnExit.getParent();
            if (ae.getActionCommand().equals("Exit"))
            {
                System.exit(0);
            }
            
            //Try and book a ticket
            if (ae.getActionCommand().equals("Book"))
            {
                System.out.println("Book was clicked");
                int custIndex = customerList.getSelectedIndex();
                int cruiseIndex = cruisesAvailable.getSelectedIndex();
                
                if (custIndex == -1 || cruiseIndex == -1)
                {
                    JOptionPane.showMessageDialog(c,
                    "You must choose a customer and a cruise");
                }else
                {
                    //Pop up a dialog asking how many tickets are requested
                    String numTickets = JOptionPane.showInputDialog(c,
                    "How many tickets?");
                    int numberOfTickets = Integer.parseInt(numTickets);
                    
                    //create a ticket request object
                    TicketRequest tickReq = new TicketRequest(custNums[custIndex],
                    lastNames[custIndex], firstNames[custIndex],
                    cruiseIDs[cruiseIndex], cruiseDestinations[cruiseIndex],
                    cruisePorts[cruiseIndex], cruiseSailings[cruiseIndex],
                    numberOfTickets);
                    
                    subject = "Request";
                    body = createEmailTicketRequest(tickReq);
                    
                    System.out.println(tickReq);
                    //Create a message using this object
                    
                    try
                    {
                        log("Sending a message to " + to);
                        
                        
                        //Create the new message
                        MimeMessage msg = new MimeMessage(session);
                        
                        //Set the send from address
                        msg.setFrom(new InternetAddress(from));
                        
                        //Set the to address
                        msg.setRecipients(Message.RecipientType.TO,
                        InternetAddress.parse(to, false));
                        
                        //Set the subject
                        msg.setSubject(subject);
                        msg.setText(body);
                        
                        //set the date
                        msg.setSentDate(new java.util.Date());
                        
                        //send the message
                        Transport.send(msg);
                        
                        System.out.println("Message was sent");
                    }catch (Exception e)
                    {
                        log("Error sending message  " + e);
                    }
                    
                    System.out.println("Sent the ticket Request Mail Message");
                }
            }
            //See if you got a mail message back
            if (ae.getActionCommand().equals("Check"))
            {
                //Check to see if there are any email messages for us
                fetchMessages();
            }
        }catch (Exception e)
        {
            System.out.println("Exception thrown = " + e);
        }
    }
    
    private String createEmailTicketRequest(TicketRequest tr)
    {
        return tr.getCustID() + "," +
        tr.getLastName() + "," +
        tr.getFirstName() + "," +
        tr.getCruiseID() + "," +
        tr.getDestination() + "," +
        tr.getPort() + "," +
        tr.getSailing() + "," +
        tr.getNumberOfTickets() ;
    }
    
    private void log(String s)
    {
        System.out.println(s);
    }
    
    private void fetchMessages()
    {
        Store store1 = null;
        Folder folder1 = null;
        try
        {
            Properties props = System.getProperties();
            Session session = Session.getDefaultInstance(props, null);
            
            //get a handle to the pop3 store
            store1 = session.getStore("pop3");
            
            //connect to the store and pass the authentication
            store1.connect(server, user, password);
            
            //obtain the default folder handle
            folder1 = store1.getDefaultFolder();
            
            //get the handle to the inbox
            folder1 = folder1.getFolder("INBOX");
            
            //if there is no inbox, throw an exception
            if (folder1 != null)
            {
                folder1.open(Folder.READ_WRITE);
            }else
                throw new Exception("no inbox available");
            
            //if there are any messages, retrieve them
            Message[] messages = folder1.getMessages();
            if (messages.length == 0)
            {
                log("There are no messages to retrieve");
            }
            
            //print out the messages
            for (int i=0; i<messages.length; i++)
            {
                logMessage(messages[i]);
            }
            
        }catch (Exception e)
        {
            log("Exception thrown in fetchMessages() " + e);
        }
        finally
        {
            try
            {
                if (folder1 != null) folder1.close(true);
                if (store1 != null) store1.close();
            }
            catch (Exception ee)
            { log("Problem closing " + ee); }
        }
    }
    
    public void logMessage(Message message1)
    {
        try
        {
            log("----------------------------------------");
            //get the From address
            InternetAddress ia = (InternetAddress)message1.getFrom()[0];
            
            //get the personal name from the message.  This is the name in
            //human readabel form like Jake Snyder.
            String personalName = ia.getPersonal();
            log("The personal name is " + personalName);
            
            //get the addressString like jakesnyder@earthlink.com
            String addressString = ia.getAddress();
            log("The address String is " + addressString);
            
            //get the contents of the subject line
            String subject = message1.getSubject();
            log("The subject is " + subject);
            if (subject.equals("Response"))
            {
                
                //Get the body of the message
                Part messagePart1 = message1;
                Object content = messagePart1.getContent();
                String contentType = messagePart1.getContentType();
                
                log("The content type is " + contentType);
                
                //If the message is plain text then you can read it with
                //a StreamReader
                if (contentType.startsWith("text/plain") ||
                contentType.startsWith("text/html"))
                {
                    InputStream is = messagePart1.getInputStream();
                    
                    BufferedReader br = new BufferedReader(new InputStreamReader(is));
                    String line1=br.readLine();
                    
                    while (line1 != null)
                    {
                        displayResult(line1);
                        line1=br.readLine();
                    }
                }
                message1.setFlag(Flags.Flag.DELETED, true);
            }
            log("----------------------------------------");
        }
        catch (Exception e)
        {
            log("Exception in logMessage " + e);
        }
    }
    
    /**
     * Message listener interface.
     * @param msg  message
     */

    // MessageListener interface
    public void displayResult(String str)
    {
        try
        {
            Container c = btnExit.getParent();

            //show the message in a dialog box
            JOptionPane.showMessageDialog(c, str);
        }catch (Exception e)
        {
            System.out.println("onMessage() Exception " + e);
        }
    }
}
