/*
 * Minimal3D.java
 *
 * Created on March 25, 2002, 9:54 AM
 */

package unleashed.ch16;

import java.awt.*;
import java.awt.event.*;
import javax.media.j3d.*;
import com.sun.j3d.utils.geometry.*;
import com.sun.j3d.utils.universe.*;
import javax.vecmath.*;



/**
 *
 * @author  Stephen Potts
 * @version
 */
public class Minimal3D_2 extends Frame
{
    
    /** Creates new Minimal3D */
    public Minimal3D_2()
    {
        super("Minimal3D_2");
        setSize(400,300);
        setLayout(new BorderLayout());
        GraphicsConfiguration config =
            SimpleUniverse.getPreferredConfiguration();

        Canvas3D canvas3D = new Canvas3D(config); 

        add("Center", canvas3D);
        setVisible(true);
        
        BranchGroup sGraph = createSceneGraph();
        sGraph.compile();
        
        SimpleUniverse sUniverse = new SimpleUniverse(canvas3D);
        
        sUniverse.getViewingPlatform().setNominalViewingTransform();
        
        sUniverse.addBranchGraph(sGraph);
    }//constructor
    
    public BranchGroup createSceneGraph()
    {
        BranchGroup root = new BranchGroup();
       
        //do a static rotation so that we can see it better
        Transform3D rotateX = new Transform3D();
        rotateX.rotX(Math.PI/10.0d);
        Transform3D rotateY = new Transform3D();
        rotateY.rotY(Math.PI/1.0d);
        rotateX.mul(rotateY);
        
        TransformGroup tGroupRotate = new TransformGroup(rotateX);
        
        //add some spinning behavior too
        TransformGroup tGroupSpin = new TransformGroup();
        tGroupSpin.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
        
        root.addChild(tGroupRotate);
        tGroupRotate.addChild(tGroupSpin);
        
        //Add the light
        addLighting(root);
        
        //add the material
        Appearance mat = new Appearance();
        Color3f darkRed = new Color3f(0.9f ,0.2f, 0.1f);
        Color3f black = new Color3f(0.0f ,0.0f, 0.0f);
        mat.setMaterial(
                 new Material(darkRed, black, darkRed, black, 1.0f));
        
        // add a simple cube
        tGroupSpin.addChild(new Box(0.3f,0.3f,0.3f, mat));
      
        //add the spin rate
        Transform3D  t3D = new Transform3D();
        Alpha rotateAlpha = new Alpha(-1,8000);
        
        RotationInterpolator rInterpolator = 
              new RotationInterpolator(rotateAlpha, tGroupSpin,
                               t3D, 0.0f, (float)Math.PI*2.0f);
        
        //add the bounds for this behavior
        BoundingSphere bSphere = new BoundingSphere();
        rInterpolator.setSchedulingBounds(bSphere);
        tGroupSpin.addChild(rInterpolator);
                
        return root;
    }//createSceneGraph()
    
    private void addLighting(BranchGroup root)
    {
        BoundingSphere bSphere = new BoundingSphere(
            new Point3d(0.0,0.0,0.0), 100.0);
        Color3f directLightColor = 
                      new Color3f(1.0f, 1.0f, 1.0f);
        
        Vector3f lightDirection = new Vector3f(1.0f, -1.0f, 0.0f);
        DirectionalLight dirLight = 
                  new DirectionalLight(directLightColor, lightDirection);
        dirLight.setInfluencingBounds(bSphere);
        root.addChild(dirLight);

    }
  
    public static void main(String[] args)
    {
        final Minimal3D_2 m3d = new Minimal3D_2();
        m3d.addWindowListener(new WindowAdapter()
        {
            public void windowClosing(WindowEvent e)
            {
                m3d.dispose();
                System.exit(0);
            }
        });
    }
}