/*
 * TextureApp.java
 *
 * Created on May 8, 2002, 9:54 AM
 */

package unleashed.ch16;

import java.awt.BorderLayout;
import java.awt.GraphicsConfiguration;
import com.sun.j3d.utils.geometry.*;
import com.sun.j3d.utils.universe.*;
import com.sun.j3d.utils.image.TextureLoader;
import javax.media.j3d.*;
import javax.vecmath.*;
import java.awt.*;
import java.awt.event.*;

/**
 * TextureApp demonstrates texture mapping.
 */
public class TextureApp extends Frame
{
    
    BranchGroup createSceneGraph()
    {
        BranchGroup objRoot = new BranchGroup();
        
        Transform3D transform = new Transform3D();
        
        QuadArray plane = new QuadArray(4, GeometryArray.COORDINATES
        | GeometryArray.TEXTURE_COORDINATE_2);
        
        Point3f p = new Point3f(-1.0f,  1.0f,  0.0f);
        plane.setCoordinate(0, p);
        p.set(-1.0f, -1.0f,  0.0f);
        plane.setCoordinate(1, p);
        p.set(1.0f, -1.0f,  0.0f);
        plane.setCoordinate(2, p);
        p.set(1.0f,  1.0f,  0.0f);
        plane.setCoordinate(3, p);
        
        Point2f q = new Point2f( 0.0f,  1.0f);
        plane.setTextureCoordinate(0, q);
        q.set(0.0f, 0.0f);
        plane.setTextureCoordinate(1, q);
        q.set(1.0f, 0.0f);
        plane.setTextureCoordinate(2, q);
        q.set(1.0f, 1.0f);
        plane.setTextureCoordinate(3, q);
        
        Appearance appear = new Appearance();
        
        String filename = "c:/unleashed/ch16/yasa.gif";
        TextureLoader loader = new TextureLoader(filename, this);
        ImageComponent2D image = loader.getImage();
        
        if(image == null)
        {
            System.out.println("load failed: "+filename);
        }
        
        Texture2D texture = new Texture2D(Texture.BASE_LEVEL, Texture.RGBA,
        image.getWidth(), image.getHeight());
        texture.setImage(0, image);
        
        appear.setTexture(texture);
        
        appear.setTransparencyAttributes(
        new TransparencyAttributes(TransparencyAttributes.FASTEST, 0.1f));
        
        Shape3D planeObj = new Shape3D(plane, appear);
        
        // rotate  the object
        Transform3D rotate = new Transform3D();
        Transform3D tempRotate = new Transform3D();
        
        rotate.rotX(Math.PI/4.0d);
        tempRotate.rotY(Math.PI/5.0d);
        rotate.mul(tempRotate);
        
        TransformGroup objRotate = new TransformGroup(rotate);
        
        TransformGroup objSpin = new TransformGroup();
        objSpin.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);

        objRoot.addChild(objRotate);
        objRotate.addChild(objSpin);
        
        objSpin.addChild(planeObj);
        
        Transform3D yAxis = new Transform3D();
        Alpha rotationAlpha = new Alpha(-1, 6000);
        
        RotationInterpolator rotator =
        new RotationInterpolator(rotationAlpha, objSpin, yAxis,
        0.0f, (float) Math.PI*2.0f);
        
        BoundingSphere bounds = new BoundingSphere();
        rotator.setSchedulingBounds(bounds);
        objSpin.addChild(rotator);
        Background background = new Background();
        background.setColor(1.0f, 1.0f, 1.0f);
        background.setApplicationBounds(new BoundingSphere());
        objRoot.addChild(background);
        
        return objRoot;
    }
    
    public TextureApp()
    {
        super("TextureApp");
        setSize(600, 400);
        setLayout(new BorderLayout());
        GraphicsConfiguration config =
        SimpleUniverse.getPreferredConfiguration();
        
        Canvas3D canvas3D = new Canvas3D(config);
        add("Center", canvas3D);
        setVisible(true);
        
        canvas3D.setStereoEnable(false);
        
        SimpleUniverse u = new SimpleUniverse(canvas3D);
        
        u.getViewingPlatform().setNominalViewingTransform();
        
        u.addBranchGraph(createSceneGraph());
    }
    
    public static void main(String[] args)
    {
        final TextureApp ta = new TextureApp();
        ta.addWindowListener(new WindowAdapter()
        {
            public void windowClosing(WindowEvent e)
            {
                ta.dispose();
                System.exit(0);
            }
        });
    }
}