package unleashed.ch21.enumeration;

import java.util.*;

/**
* Traffic Signal is an enumeration of traffic signals.
*/
class TrafficSignal
{
  	private final String name;
	
	//Ordinal of next type is created
    private static int nextOrdinal = 0;
	
	//Assign an ordinal to this type
	private final int ordinal = nextOrdinal++; 



	protected TrafficSignal(String name)
	{
		this.name = name;
	}

	public String getName()
	{
		return name;
	}

	
	public static final TrafficSignal GREEN= new TrafficSignal("GREEN");

	public static final TrafficSignal YELLOW = new TrafficSignal("YELLOW");

	public static final TrafficSignal RED = new TrafficSignal("RED");
	
	
	/**
	* Overrides the Object equals method.
	*/
	public boolean equals(Object o)
	{
		return ordinal == ((TrafficSignal)o).ordinal;
	}

    
}

/**
* Traffic simulator.
*/

public class Traffic
{
	/**
	* Traffic moves according to the signal.
	*/
	public String move(TrafficSignal light)
	{
		if(light.equals(TrafficSignal.GREEN))
			return "Signal is green. Traffic is moving.";
		else if(light.equals(TrafficSignal.YELLOW))
			return "Signal is yellow. Traffic is stopping.";
		else if(light.equals(TrafficSignal.RED))
			return "Signal is red. Traffic is waiting.";
		else
			return "UNKNOWN SIGNAL";	
	}
	
	public static void main(String[] args)
	{
		Traffic traffic = new Traffic();
		
		System.out.println("Starting the traffic signal");
		
		System.out.println(traffic.move(TrafficSignal.GREEN));
		System.out.println(traffic.move(TrafficSignal.YELLOW));
		System.out.println(traffic.move(TrafficSignal.RED));
		
		System.out.println("Shutting down the signal");
		
		
	}

}






