package unleashed.ch21.factory;

/**
* Factory pattern example
* Abstract connection factory
*/
interface DataConnectionFactory
{
	/**
	* Gets connection.
	*/
	public DataConnection getConnection();

}

/**
* Abstract data storage connection.
*/
interface DataConnection
{
	/**
	* Connection can read data.
	*/
	public Object readData();
	
	/**
	* Connection can write data.
	*/
	public void writeData(Object object);
} 

/**
* Implements connection factory to for a fake connection.
*/
class FakeConnectionFactory implements DataConnectionFactory
{
	public FakeConnectionFactory()
	{
        System.out.println("Creating fake connection factory");
    }
	//factory specific field
	protected String filename = "datafile.in";

	public DataConnection getConnection()
	{
		//open file for reading
		return new FakeDataConnection();
	}
}

/**
* Implements connection to fake data
*/
class FakeDataConnection implements DataConnection
{
	public FakeDataConnection()
	{
	    System.out.println("Creating fake data connection");
	}
	
	private String fakeData = "Example fake data";
	
	public Object readData()
	{
		System.out.println("Reading data from fake data connection");
		return fakeData;
	}
	
	public void writeData(Object object)
	{
		fakeData = object.toString();
	}
}

/**
* Implements connection factory for database connection
*/
class DatabaseConnecionFactory implements DataConnectionFactory
{
	//factory specific field
	protected String databaseConnectionURL = "database connection string";

	public DataConnection getConnection()
	{
		//code to open connection
		return null;
	}
}

/**
* Client creates a connection factory, gets connection and reads data from the
* connection
*/
public class FactoryExample
{
	
	protected static DataConnectionFactory dataFactory = new FakeConnectionFactory();
	
	//protected static DataConnectionFactory dataFactory = new DatabaseConnectionFactory();

	public static void main(String[] args)
	{
		//get connection to data
		DataConnection connection = dataFactory.getConnection();
		
		//read data
		Object mydata = connection.readData();
		
		//print the object
		System.out.println("Read data: " + mydata.toString());
	}
}
