package unleashed.ch23;

import java.io.*;
import java.net.*;
import javax.net.*;
import javax.net.ssl.*;


/**
* SSLClientExample creates an SSL socket to connect to local Weblogic server. 
*/
public class SSLClientExample
{
    /**
    * Creates SSL socket, connects to the server, 
    * sends data and receives server response.
    */
	public static void main(String[] args)
	{
		try
		{
        
		//create SSL socket factory
        SocketFactory sf = SSLSocketFactory.getDefault();
		
        //create SSL socket
		SSLSocket s = (SSLSocket)sf.createSocket("localhost", Integer.parseInt("9096"));
        
		System.out.println("Client started");
        
        //enable cipher suite
		enableAllCipherSuites(s);
		
        //create reader
		BufferedReader br =  new BufferedReader(
			new InputStreamReader(s.getInputStream() ));
	    
        //create writer
		PrintWriter pw = new PrintWriter(s.getOutputStream());
                
        System.out.println("Connection established");
        
        System.out.println("Sending data");
		
        //send data
       	pw.println("HELLO FROM SSL CLIENT");
        
        //need to flush the buffer or the data is not sent
	    pw.flush();
        
        System.out.println("Data is sent...receiving a response from the server");    
        
        //read the data from the server
		System.out.println("Client received data: " + br.readLine());
		System.out.println("Closing connection");
        
        //close socket
		s.close();
		
		}catch(Exception e)
		{
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
	}
	
    /**
    * Enable the cipher suite that is used in JDK truststore
    */
	private static void enableAllCipherSuites(SSLSocket ss)
	{
			String newsuites[] = new String[8];
	
    		String suites[] = ss.getEnabledCipherSuites();
	
    		suites[0] = "SSL_DH_anon_EXPORT_WITH_DES40_CBC_SHA";
			ss.setEnabledCipherSuites(suites);
			
	}
}
